<template>
    <div>
        <div class="module-cart-billing block-billing">
            FACTURACIÓN <span>{{ formatCurrency(subtotal) }}</span>
        </div>
        <BillingMethod ref="billingMethodRef" />
        <hr class="hr-divisor" />
        <div class="module-cart-billing">
            ENVÍO<span>{{ formatCurrencyShipping(shippingCost) }}</span>
        </div>
        <ShippingFull ref="shippingMethodRef" />
        <hr class="hr-divisor" />
        <div class="module-cart-billing">
            MÉTODO DE PAGO<span>{{ formatCurrencyPayment(paymentDiscount) }}</span>
        </div>
        <PaymentMethod
            ref="paymentMethodRef"
            :from="'billing'"
            :payment-methods="paymentMethods"
            @payment-failed="handlePaymentFail"
        />
        <hr class="hr-divisor" />
        <div class="module-cart-billing">
            NOTAS
            <div>
                <textarea id="notes" v-model="store.notes" class="form-control"></textarea>
            </div>
        </div>
        <hr class="hr-divisor" />
        <TotalsCart />

        <button class="just-button total-button" :disabled="isSubmitting" @click="finalizePurchase">
            <span v-if="!isSubmitting">Finalizar Compra</span>
            <span v-else class="spinner"></span>
        </button>
    </div>
</template>

<script setup>
import { computed, ref, watch } from "vue";
import BillingMethod from "./BillingMethod.vue";
import ShippingFull from "./ShippingFull.vue";
import { cartStore } from "../../../js/stores/cart";
import TotalsCart from "../TotalsCart.vue";
import PaymentMethod from "../Summary/PaymentMethod.vue";
import { sendOrderData } from "../../../js/services/orderService.js";

const store = cartStore();

const billingMethodRef = ref(null);
const shippingMethodRef = ref(null);
const paymentMethodRef = ref(null);

const notes = ref(store.notes);

const isFormBillingValid = ref(false);
const isFormShippingValid = ref(false);
const isSubmitting = ref(false);

const subtotal = computed(() => store.subtotal);
const shippingCost = computed(() => store.getShippingCost());
const paymentDiscount = computed(() => store.getPaymentDiscount());

watch(shippingCost, () => store.getShippingCost());
watch(paymentDiscount, () => store.getPaymentDiscount());
watch(notes, (value) => store.updateNotes(value));

function handlePaymentFail() {
    // Escucho si PaymentMethod emite error en caso de fallo de mercado pago para q me permita hacer submit nuevamente
    isSubmitting.value = false;
}

const formatCurrency = (value) => {
    return new Intl.NumberFormat("es-AR", {
        style: "currency",
        currency: "ARS",
        minimumFractionDigits: 2,
        maximumFractionDigits: 2,
    }).format(value);
};

const formatCurrencyShipping = (value) => {
    if (value === 0) {
        return "Gratis";
    }
    return new Intl.NumberFormat("es-AR", {
        style: "currency",
        currency: "ARS",
        minimumFractionDigits: 2,
        maximumFractionDigits: 2,
    }).format(value);
};

const formatCurrencyPayment = (value) => {
    if (value === 0) {
        return "";
    }
    return (
        "-" +
        new Intl.NumberFormat("es-AR", {
            style: "currency",
            currency: "ARS",
            minimumFractionDigits: 2,
            maximumFractionDigits: 2,
        }).format(value)
    );
};

async function finalizePurchase() {
    if (isSubmitting.value) return;

    if (!store.items.length) {
        window.location.href = "/tienda";
        return;
    }

    isFormBillingValid.value = billingMethodRef.value.validateBillingForm();
    isFormShippingValid.value = shippingMethodRef.value.validateShippingForm();

    if (isFormBillingValid.value && isFormShippingValid.value) {
        isSubmitting.value = true;

        if (store.selectedPaymentMethod === "mercado_pago") {
            try {
                await paymentMethodRef.value.submitMercadoPago();
            } catch (error) {
                console.error("Error durante el pago con Mercado Pago:", error);
            }
            return;
        }

        // Para otros métodos de pago, enviás la orden directamente
        const order = {
            items: store.items,
            shipping: store.selectedShippingMethod,
            payment: store.selectedPaymentMethod,

            billingName: store.billingName,
            billingSurname: store.billingSurname,
            billingDni: store.billingDni,
            billingEmail: store.billingEmail,
            billingPhone: store.billingPhone,

            billingCity: store.billingCity,
            billingAddress: store.billingAddress,
            billingNumber: store.billingNumber,
            billingDpto: store.billingDpto,
            billingFloor: store.billingFloor,
            billingPostalCode: store.billingPostalCode,

            wantInvoice: store.wantInvoice,
            invoicingCuit: store.invoicingCuit,
            invoicingCompanyName: store.invoicingCompanyName,
            invoicingAddress: store.invoicingAddress,
            invoicingCity: store.invoicingCity,
            address: store.address,
            number: store.number,
            dpto: store.dpto,
            floor: store.floor,
            receiveDni: store.receiveDni,
            receiveEmail: store.receiveEmail,
            receiveName: store.receiveName,
            receiveSurname: store.receiveSurname,
            receivePhone: store.receivePhone,
            selectedBranch: store.selectedBranch,
            selectedCity: store.selectedCity,
            shippingPostalCode: store.shippingPostalCode,
            selectedProvince: store.selectedProvince,
            changeShipping: store.changeShipping,
            notes: store.notes,
            extraData: store.extraData,
        };

        try {
            await sendOrderData(order, store);
        } catch (error) {
            console.error("Error al enviar los datos:", error);
            alert("Hubo un error al procesar la compra.");
        } finally {
            isSubmitting.value = false;
        }
    }
}

const props = defineProps({
    paymentMethods: {
        type: Array,
        required: true,
    },
});
</script>

<style lang="scss">
@use "../../../scss/variables" as vars;

.block-billing {
    margin-top: 10px;
}

.module-cart-billing {
    text-transform: uppercase;
    font-family: "Plus Jakarta Sans", sans-serif;
    font-weight: 800;
    padding-bottom: 20px;

    span {
        float: right;
        font-size: 16px;
    }
}

.total-button[disabled] {
    border: 1px solid #999 !important;
    background-color: #565656 !important;
}

#notes {
    resize: none;
}

.spinner {
    display: inline-block;
    width: 20px;
    height: 20px;
    border: 3px solid rgb(255 255 255 / 30%);
    border-top-color: white;
    border-radius: 50%;
    animation: spin 1s linear infinite;
    vertical-align: middle;
}

@keyframes spin {
    to {
        transform: rotate(360deg);
    }
}
</style>
